@@ -15,7 +15,8 @@
 
 import contextlib
 import errno
-import hashlib
+import io
+import math
 import os
 from unittest import mock
 
@@ -30,6 +31,7 @@ import fixtures
 from os_brick.initiator import connector
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 
 import glance_store as store
@@ -87,7 +89,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         fake_sc = [{u'endpoints': [{u'publicURL': u'http://foo/public_url'}],
                     u'endpoints_links': [],
                     u'name': u'cinder',
-                    u'type': u'volumev2'}]
+                    u'type': u'volumev3'}]
         self.context = FakeObject(service_catalog=fake_sc,
                                   user_id='fake_user',
                                   auth_token='fake_token',
@@ -219,7 +221,18 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                 mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value=root_helper), \
                 mock.patch.object(connector.InitiatorConnector, 'factory',
-                                  side_effect=fake_factory):
+                                  side_effect=fake_factory
+                                  ) as fake_conn_obj, \
+                mock.patch.object(socket,
+                                  'gethostname') as mock_get_host, \
+                mock.patch.object(socket,
+                                  'getaddrinfo') as mock_get_host_ip:
+
+            fake_host = 'fake_host'
+            fake_addr_info = [[0, 1, 2, 3, ['127.0.0.1']]]
+            fake_ip = fake_addr_info[0][4][0]
+            mock_get_host.return_value = fake_host
+            mock_get_host_ip.return_value = fake_addr_info
 
             with mock.patch.object(connector,
                                    'get_connector_properties') as mock_conn:
@@ -229,8 +242,8 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                     do_open()
 
                 mock_conn.assert_called_once_with(
-                    root_helper, socket.gethostname(), multipath_supported,
-                    enforce_multipath)
+                    root_helper, fake_ip, multipath_supported,
+                    enforce_multipath, host=fake_host)
                 fake_connector.connect_volume.assert_called_once_with(mock.ANY)
                 fake_connector.disconnect_volume.assert_called_once_with(
                     mock.ANY, fake_devinfo)
@@ -238,6 +251,9 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                     None, 'glance_store', attach_mode,
                     host_name=socket.gethostname())
                 fake_volumes.detach.assert_called_once_with(fake_volume)
+                fake_conn_obj.assert_called_once_with(
+                    mock.ANY, root_helper, conn=mock.ANY,
+                    use_multipath=multipath_supported)
 
     def test_open_cinder_volume_rw(self):
         self._test_open_cinder_volume('wb', 'rw', None)
@@ -271,27 +287,55 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
 
         self.store._check_context(FakeObject(service_catalog='fake'))
 
-    def test_cinder_configure_add(self):
+    def test_configure_add(self):
+
+        def fake_volume_type_check(name):
+            if name != 'some_type':
+                raise cinder.cinder_exception.NotFound(code=404)
+
         with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
-            def raise_(ex):
-                raise ex
             mocked_cc.return_value = FakeObject(volume_types=FakeObject(
-                find=lambda name: 'some_type' if name == 'some_type'
-                else raise_(cinder.cinder_exception.NotFound(code=404))))
+                find=fake_volume_type_check))
             self.config(cinder_volume_type='some_type',
                         group=self.store.backend_group)
             # If volume type exists, no exception is raised
             self.store.configure_add()
-            # setting cinder_volume_type to non-existent value will raise
-            # BadStoreConfiguration exception
+            # setting cinder_volume_type to non-existent value will log a
+            # warning
             self.config(cinder_volume_type='some_random_type',
                         group=self.store.backend_group)
+            with mock.patch.object(cinder, 'LOG') as mock_log:
+                self.store.configure_add()
+                mock_log.warning.assert_called_with(
+                    "Invalid `cinder_volume_type some_random_type`")
 
-            self.assertRaises(exceptions.BadStoreConfiguration,
-                              self.store.configure_add)
-            # when only 1 store is configured, BackendException is raised
-            self.config(enabled_backends={'cinder1': 'cinder'})
-            self.assertRaises(exceptions.BackendException,
+    def test_configure_add_cinder_service_down(self):
+
+        def fake_volume_type_check(name):
+            raise cinder.cinder_exception.ClientException(code=503)
+
+        self.config(cinder_volume_type='some_type',
+                    group=self.store.backend_group)
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volume_types=FakeObject(
+                find=fake_volume_type_check))
+            # We handle the ClientException to pass so no exception is raised
+            # in this case
+            self.store.configure_add()
+
+    def test_configure_add_authorization_failed(self):
+
+        def fake_volume_type_check(name):
+            raise cinder.exceptions.AuthorizationFailure(code=401)
+
+        self.config(cinder_volume_type='some_type',
+                    group=self.store.backend_group)
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volume_types=FakeObject(
+                find=fake_volume_type_check))
+            # Anything apart from invalid volume type or cinder service
+            # down will raise an exception
+            self.assertRaises(cinder.exceptions.AuthorizationFailure,
                               self.store.configure_add)
 
     def test_is_image_associated_with_store(self):
@@ -398,12 +442,13 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             self.assertEqual(expected_image_size, image_size)
 
     def _test_cinder_add(self, fake_volume, volume_file, size_kb=5,
-                         verifier=None, backend="cinder1"):
+                         verifier=None, backend="cinder1", fail_resize=False):
         expected_image_id = str(uuid.uuid4())
         expected_size = size_kb * units.Ki
         expected_file_contents = b"*" * expected_size
         image_file = six.BytesIO(expected_file_contents)
-        expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_checksum = md5(expected_file_contents,
+                                usedforsecurity=False).hexdigest()
         expected_location = 'cinder://%s/%s' % (backend, fake_volume.id)
         fake_client = FakeObject(auth_token=None, management_url=None)
         fake_volume.manager.get.return_value = fake_volume
@@ -417,7 +462,11 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
 
         with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
-                                  side_effect=fake_open):
+                                  side_effect=fake_open), \
+                mock.patch.object(
+                    cinder.Store, '_wait_resize_device') as mock_wait_resize:
+            if fail_resize:
+                mock_wait_resize.side_effect = exceptions.BackendException()
             mock_cc.return_value = FakeObject(client=fake_client,
                                               volumes=fake_volumes)
             loc, size, checksum, metadata = self.store.add(expected_image_id,
@@ -492,3 +541,43 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                                      size=1)
         volume_file = six.BytesIO()
         self._test_cinder_add(fake_volume, volume_file, backend="cinder2")
+
+    def test__get_device_size(self):
+        fake_data = b"fake binary data"
+        fake_len = int(math.ceil(float(len(fake_data)) / units.Gi))
+        fake_file = io.BytesIO(fake_data)
+        dev_size = cinder.Store._get_device_size(fake_file)
+        self.assertEqual(fake_len, dev_size)
+
+    @mock.patch.object(time, 'sleep')
+    def test__wait_resize_device_resized(self, mock_sleep):
+        fake_vol = mock.MagicMock()
+        fake_vol.size = 2
+        fake_file = io.BytesIO(b"fake binary data")
+        with mock.patch.object(
+                cinder.Store, '_get_device_size') as mock_get_dev_size:
+            mock_get_dev_size.side_effect = [1, 2]
+            cinder.Store._wait_resize_device(fake_vol, fake_file)
+
+    @mock.patch.object(time, 'sleep')
+    def test__wait_resize_device_fails(self, mock_sleep):
+        fake_vol = mock.MagicMock()
+        fake_vol.size = 2
+        fake_file = io.BytesIO(b"fake binary data")
+        with mock.patch.object(
+                cinder.Store, '_get_device_size',
+                return_value=1):
+            self.assertRaises(
+                exceptions.BackendException,
+                cinder.Store._wait_resize_device,
+                fake_vol, fake_file)
+
+    def test_cinder_add_fail_resize(self):
+        volume_file = io.BytesIO()
+        fake_volume = mock.MagicMock(id=str(uuid.uuid4()),
+                                     status='available',
+                                     size=1)
+        self.assertRaises(exceptions.BackendException,
+                          self._test_cinder_add, fake_volume, volume_file,
+                          fail_resize=True)
+        fake_volume.delete.assert_called_once()
